<?php

/**
 * CustomIcon
 *
 * @link       https://appcheap.io
 * @since      1.0.0
 * @author     ngocdt
 * @package    AppBuilder
 */

namespace AppBuilder\Feature;

defined( 'ABSPATH' ) || exit;

/**
 * CustomIcon Class.
 */

if ( ! class_exists( 'CustomIcon' ) ) {
	/**
	 * Class CustomIcon
	 */
	class CustomIcon {

		/**
		 * Meta key
		 *
		 * @var string
		 */
		private $meta_key = 'app_builder_custom_icon_settings';

		/**
		 * Post_Types constructor.
		 */
		public function __construct() {
			$this->register_hooks();
		}

		/**
		 * Get meta key
		 */
		public function get_meta_key() {
			return $this->meta_key;
		}

		/**
		 * Get settings
		 */
		public function get_settings() {
			$data = $this->get_url_icons();
			return array(
				array(
					'title'  => 'Custom Icon',
					'fields' => array(
						array(
							'name'  => 'icons',
							'label' => 'Add icon links',
							'hint'  => 'You can put multi icon urls with .json and .css file generate in admin.appcheap.io and separate by new line.',
							'type'  => 'textarea',
							'value' => $data,
						),
					),
				),
			);
		}

		/**
		 * Get settings
		 *
		 * @return array settings
		 */
		public function get_data() {
			$data = get_option( $this->meta_key, array() );

			if ( ! is_array( $data ) ) {
				$data = array();
			}

			return $data;
		}

		/**
		 * Get url icons
		 *
		 * @return string
		 */
		public function get_url_icons() {
			$data = $this->get_data();

			$icons = isset( $data['icons'] ) ? $data['icons'] : '';

			// Check not empty string.
			if ( empty( $icons ) ) {
				return '';
			}

			return $icons;
		}

		/**
		 * Register hooks
		 */
		public function register_hooks() {
			add_filter( 'app_builder_features', array( $this, 'app_builder_features' ), 1, 1 );
			add_filter( 'app_builder_custom_css', array( $this, 'custom_css' ), 10, 1 );
			add_filter( 'app_builder_custom_data', array( $this, 'custom_data' ), 10, 1 );
			add_action( 'app_builder_features_post', array( $this, 'app_builder_features_post' ), 10, 1 );
		}

		/**
		 * App builder features post
		 *
		 * @param array $data data.
		 *
		 * @return void
		 */
		public function app_builder_features_post( $data ) {
			if ( isset( $data[ $this->meta_key ] ) ) {
				update_option( $this->meta_key, $data[ $this->meta_key ] );
			}
		}

		/**
		 * Custom css
		 *
		 * @param array $css css.
		 *
		 * @return array
		 */
		public function custom_css( $css ) {

			$icons = $this->get_url_icons();

			if ( empty( $icons ) ) {
				return $css;
			}

			// Init time.
			$custom_css = is_array( $css ) ? $css : array();

			// Parse icons.
			$list_css_urls = $this->parse_urls( $icons, 'css' );

			foreach ( $list_css_urls as $css_url ) {
				$query = wp_parse_url( $css_url, PHP_URL_QUERY );
				parse_str( $query, $params );
				$custom_css[] = array(
					'name'    => md5( $css_url ),
					'url'     => $css_url,
					'version' => $params['v'],
				);
			}

			return $custom_css;
		}

		/**
		 * Custom data
		 *
		 * @param array $data data.
		 *
		 * @return array
		 */
		public function custom_data( $data = array() ) {
			$icons = $this->get_url_icons();

			if ( empty( $icons ) ) {
				return $data;
			}

			$custom_json = array();

			// Parse icons.
			$icon_json_urls = $this->parse_urls( $icons, 'json' );

			foreach ( $icon_json_urls as $json_url ) {
				// Use wp_remote_get instead of file_get_contents.
				$json_data = wp_remote_get( $json_url );
				if ( is_wp_error( $json_data ) ) {
					continue;
				}

				$json_data_body = wp_remote_retrieve_body( $json_data );
				if ( empty( $json_data_body ) ) {
					continue;
				}

				$json_data_body_decoded = json_decode( $json_data_body, true );
				if ( empty( $json_data_body_decoded ) ) {
					continue;
				}

				// Validate json data has fontName and icon property.
				if ( ! isset( $json_data_body_decoded['fontName'] ) || ! isset( $json_data_body_decoded['icons'] ) ) {
					continue;
				}

				$custom_json[] = $json_data_body_decoded;
			}

			$data['custom_icons'] = wp_json_encode( $custom_json );

			return $data;
		}

		/**
		 * Get custom icon settings.
		 *
		 * @param array $features features.
		 *
		 * @return array
		 */
		public function app_builder_features( $features = array() ) {
			$features[ $this->meta_key ] = $this->get_settings();
			return $features;
		}

		/**
		 * Admin permissions check
		 *
		 * @return bool
		 */
		public function admin_permissions_check() {
			return current_user_can( 'manage_options' );
		}

		/**
		 * Parse urls
		 *
		 * @param string $urls urls.
		 * @param string $mime_type mime type.
		 *
		 * @return array
		 */
		public function parse_urls( $urls, $mime_type = 'css' ) {
			if ( empty( $urls ) ) {
				return array();
			}

			// Parse by new line.
			$urls = explode( "\n", $urls );

			$result = array();
			foreach ( $urls as $url ) {
				$url = trim( $url );
				if ( empty( $url ) ) {
					continue;
				}

				// Continue if not valid url.
				if ( filter_var( $url, FILTER_VALIDATE_URL ) === false ) {
					continue;
				}

				// Continue if not css file.
				if ( pathinfo( wp_parse_url( $url, PHP_URL_PATH ), PATHINFO_EXTENSION ) !== $mime_type ) {
					continue;
				}

				$result[] = $url;
			}

			return $result;
		}
	}
}
